@@ -1,11 +1,17 @@
 import re
 import string
 import sys
-from typing import Any, Iterable, List, Tuple, Union
 import unicodedata
+from typing import Callable, Iterable, Union
+
+from d8s_dicts import dict_delistify_values, dict_flip
+from d8s_lists import deduplicate, has_index, shortest, truthy_items
+from d8s_math import number_evenly_divides, percent
 
 # from textblob import TextBlob
 
+# pylint: disable=C0415
+
 
 # TODO: add a function to get a substring between two given characters
 # TODO: write function to split a given string up into subparts of a given length
@@ -13,6 +19,20 @@ import unicodedata
 NO_ISASCII_AVAILABLE = sys.version_info.major == 3 and sys.version_info.minor <= 6
 
 
+def string_modify_line(input_string: str, modifying_func: Callable[[str], str], line_num: int) -> str:
+    """Apply the modifying_func on the input_string at the given line_num."""
+    if line_num < 1:
+        raise ValueError(
+            'Please provide a line_num >= 1. The line number is NOT zero indexed - so a line_num of one specifies the first line of the text.'
+        )
+
+    updated_line_num = line_num - 1
+
+    lines = input_string.splitlines()
+    lines[updated_line_num] = modifying_func(lines[updated_line_num])
+    return '\n'.join(lines)
+
+
 def string_chars_at_start(string: str, chars: Iterable) -> Iterable[str]:
     """."""
     for char in string:
@@ -118,8 +138,8 @@ def pluralize(word: str) -> str:
 def is_singular(possible_singular: str) -> bool:
     """Return whether or not the possible_singular is singular."""
     # this is a repetition of the code from the is_plural function and does not simply return `not is_plural` because...
-    # there are many different responses possible from inflect_engine.compare and there are cases where inflect_engine.compare...
-    # cannot compare the two words
+    # there are many different responses possible from inflect_engine.compare and there are cases where
+    # inflect_engine.compare... cannot compare the two words
     singular = False
     inflect_engine = _inflect_engine()
     pluralized_word = inflect_engine.plural(possible_singular)
@@ -149,7 +169,8 @@ def cardinalize(word: str, count: int) -> str:
         if count == 1:
             return word
         word = pluralize(word)
-    # I know this is using the singular_noun function, but it will return either singular or plural nouns based on the count argument
+    # I know this is using the singular_noun function, but it will return either singular or plural nouns
+    # based on the count argument
     return inflect_engine.singular_noun(word, count=count)
 
 
@@ -164,8 +185,7 @@ def string_forms(text):
     # it is important to lowercase the text before we start so that we can avoid problems when making the text plural
     text = lowercase(text)
 
-    # TODO: may want to standardize casing of the keys... `uppercase` vs. `kebab_case`... either `uppercase` should be `upperCase` or `kebab_case` should be `kebabcase`
-    string_forms = {
+    forms = {
         'lowercase': lowercase(text),
         'titlecase': titlecase(text),
         'uppercase': uppercase(text),
@@ -185,7 +205,7 @@ def string_forms(text):
         'uppercaseIndefiniteArticle': uppercase(indefinite_article(text)),
     }
 
-    return string_forms
+    return forms
 
 
 def _inflect_engine():
@@ -197,15 +217,15 @@ def _inflect_engine():
 
 
 def string_left_pad(string, length: int, *, padding_characters=' '):
-    """Pad the given string with the given padding_characters such that the length of the resulting string is equal to the `length` argument. Adapted from the javascript code here: https://www.theregister.co.uk/2016/03/23/npm_left_pad_chaos/."""
-    from .strings_temp_utils import number_evenly_divides
+    """Pad the string with padding_characters such that the resulting string is the given length.
 
+    Adapted from the javascript code here: https://www.theregister.co.uk/2016/03/23/npm_left_pad_chaos/."""
     padding_length = length - len(string)
 
     if padding_length and not number_evenly_divides(len(padding_characters), padding_length):
         message = f'The length of the padding_characters ({len(padding_characters)}) must evenly divide the desired length of the final string ({length}).'
         raise ValueError(message)
-    else:
+    else:  # pylint: disable=R1720
         padding_length = int(padding_length / len(padding_characters))
 
     left_padded_string = padding_characters * padding_length + string
@@ -223,9 +243,8 @@ def string_to_bool(string: str) -> bool:
 
 def text_examples(n=10):
     """Create n example texts."""
-    from hypothesis.strategies import text
-
     from d8s_hypothesis import hypothesis_get_strategy_results
+    from hypothesis.strategies import text
 
     return hypothesis_get_strategy_results(text, n=n)
 
@@ -239,9 +258,8 @@ def string_has_multiple_consecutive_spaces(string):
 
 def character_examples(n=10):
     """Create n example characters."""
-    from hypothesis.strategies import characters
-
     from d8s_hypothesis import hypothesis_get_strategy_results
+    from hypothesis.strategies import characters
 
     return hypothesis_get_strategy_results(characters, n=n)
 
@@ -274,10 +292,7 @@ def text_input_is_no(message):
 
 def string_is_yes(string):
     """Check if a string is some form of `y` or `yes`."""
-    if lowercase(string) == 'y' or lowercase(string) == 'yes':
-        return True
-    else:
-        return False
+    return bool(lowercase(string) == 'y' or lowercase(string) == 'yes')
 
 
 def string_is_no(string):
@@ -399,13 +414,11 @@ def strings_diff_opcodes(a: str, b: str):
     """Return the opcodes representing the differences/similarities between two strings."""
     sequence_matcher = string_sequence_matcher(a, b)
 
-    return [i for i in sequence_matcher.get_opcodes()]
+    return sequence_matcher.get_opcodes()
 
 
 def string_common_prefix(a: str, b: str) -> str:
     """Returns the common prefix string from left to right between a and b."""
-    from .strings_temp_utils import shortest
-
     common_prefix = ''
 
     for index in range(len(shortest([a, b]))):
@@ -455,17 +468,15 @@ def unicode_number_to_character(unicode_number):
 
 def hamming_distance(string_1, string_2, as_percent=False):
     """Return the number of positions at which corresponding symbols in string_1 and string_2 are different (this is known as the Hamming Distance). See https://en.wikipedia.org/wiki/Hamming_distance."""
-    from .strings_temp_utils import percent
-
     if len(string_1) != len(string_2):
         raise ValueError('The length of the two strings must be the same')
 
-    hamming_distance = sum(el1 != el2 for el1, el2 in zip(string_1, string_2))
+    distance = sum(el1 != el2 for el1, el2 in zip(string_1, string_2))
 
     if as_percent:
-        return percent(hamming_distance / len(string_1))
+        return percent(distance / len(string_1))
     else:
-        return hamming_distance
+        return distance
 
 
 def from_char_code(integer_list):
@@ -473,7 +484,7 @@ def from_char_code(integer_list):
     return ''.join([chr(int(integer)) for integer in integer_list])
 
 
-def text_ascii_characters(text: str) -> Tuple[str]:
+def text_ascii_characters(text: str) -> Iterable[str]:
     """."""
     if NO_ISASCII_AVAILABLE:
         for char in text:
@@ -489,7 +500,7 @@ def text_ascii_characters(text: str) -> Tuple[str]:
                 yield char
 
 
-def text_non_ascii_characters(text: str) -> Tuple[str]:
+def text_non_ascii_characters(text: str) -> Iterable[str]:
     """."""
     if NO_ISASCII_AVAILABLE:
         for char in text:
@@ -520,8 +531,6 @@ def string_entropy(text, ignore_case=False):
     """Find the shannon entropy of the text. Inspired by the algorithm here https://web.archive.org/web/20160320142455/https://deadhacker.com/2007/05/13/finding-entropy-in-binary-files/. You can see more here: https://en.wikipedia.org/wiki/Entropy_(information_theory)"""
     import math
 
-    from .strings_temp_utils import deduplicate
-
     if ignore_case:
         text = text.lower()
 
@@ -544,19 +553,19 @@ def substrings(iterable):
     return more_itertools.substrings(iterable)
 
 
-def string_remove_non_alphabetic_characters(string: str):
-    """."""
-    pass
+# def string_remove_non_alphabetic_characters(string: str):
+#     """."""
+#     pass
 
 
-def string_remove_non_numeric_characters(string: str):
-    """."""
-    pass
+# def string_remove_non_numeric_characters(string: str):
+#     """."""
+#     pass
 
 
 def string_remove_non_alpha_numeric_characters(string: str):
     """."""
-    pattern = '[^a-zA-Z\d\s]'
+    pattern = r'[^a-zA-Z\d\s]'
     string_after_removal = string_remove(pattern, string)
     return string_after_removal
 
@@ -577,7 +586,7 @@ def string_remove_unicode(string: str):
 
 def string_remove_numbers(input_string: str, replacement: str = ' '):
     """Remove all numbers from the input_strings."""
-    new_string_without_numbers = re.sub('\d+', replacement, input_string)
+    new_string_without_numbers = re.sub(r'\d+', replacement, input_string)
     return new_string_without_numbers
 
 
@@ -668,24 +677,17 @@ def string_shorten(input_string, length, suffix='...'):
 
 def string_split_without_empty(input_string, split_char):
     """Split a input_string on split_char and remove empty entries."""
-    from .strings_temp_utils import list_delete_empty_items
-
-    return list_delete_empty_items(input_string.split(split_char))
+    return truthy_items(input_string.split(split_char))
 
 
 def string_has_index(string: str, index: Union[str, int]) -> bool:
     """."""
-    from .strings_temp_utils import list_has_index
-
     string_characters = characters(string)
-    has_index = list_has_index(string_characters, index)
-    return has_index
+    return has_index(string_characters, index)
 
 
 def string_split_on_uppercase(input_string: str, include_uppercase_characters=False, split_acronyms=True):
     """Split the input_string on uppercase characters. If split_acronyms is False, the function will not split consecutive uppercase letters."""
-    from .strings_temp_utils import list_delete_empty_items
-
     if not split_acronyms and not include_uppercase_characters:
         message = 'If you set the `split_acronyms` to False when calling the `string_split_on_uppercase` function, you must also set the `include_uppercase_characters` (which you did not). The function will continue, but the `split_acronyms` argument will make no difference.'
         raise ValueError(message)
@@ -715,14 +717,12 @@ def string_split_on_uppercase(input_string: str, include_uppercase_characters=Fa
 
     split_string.append(input_string[last_uppercase_character_index:])
 
-    # we are using the list_delete_empty_items function b/c if include_uppercase_characters is False and the last character of the input_string is uppercase, an empty string will be erroneously included in the response from this function
-    return list_delete_empty_items(split_string)
+    # we are using the truthy_items function b/c if include_uppercase_characters is False and the last character of the input_string is uppercase, an empty string will be erroneously included in the response from this function
+    return truthy_items(split_string)
 
 
 def string_split_on_lowercase(input_string, include_lowercase_characters=False):
     """Split the string on lowercase characters."""
-    from .strings_temp_utils import list_delete_empty_items
-
     split_string = []
     last_lowercase_character_index = 0
 
@@ -740,7 +740,7 @@ def string_split_on_lowercase(input_string, include_lowercase_characters=False):
     split_string.append(input_string[last_lowercase_character_index:])
 
     # see the note from the string_split_on_uppercase function
-    return list_delete_empty_items(split_string)
+    return truthy_items(split_string)
 
 
 def string_split_multiple(string, *splitting_characters):
@@ -934,15 +934,17 @@ def _handle_casing(item, casing):
             casing, available_casing_types
         )
         raise ValueError(message)
-    if isinstance(item, str) or isinstance(item, bytes):
-        return eval('item.{}()'.format(casing))
+    if isinstance(item, (str, bytes)):
+        return eval('item.{}()'.format(casing))  # pylint: disable=W0123
     else:
         print('! Democritus cannot yet {}-case an item of type {}'.format(casing, type(item)))
         return item
 
 
 def string_rotate(text, rot=13):
-    """Return the text converted using a Caesar cipher (https://en.wikipedia.org/wiki/Caesar_cipher) in which the text is rotated by the given amount (using the `rot` argument)."""
+    """Return the text converted using a Caesar cipher in which the text is rotated by the given amount.
+
+    See https://en.wikipedia.org/wiki/Caesar_cipher for more details."""
     # credit for the algorithm: https://github.com/python/cpython/blob/master/Lib/this.py
     d = {}
     for c in (65, 97):
@@ -952,14 +954,14 @@ def string_rotate(text, rot=13):
     return "".join([d.get(c, c) for c in text])
 
 
-def text_is_english_sentence(text: str) -> bool:
-    """Determine whether or not the sentence is likely English."""
-    language_detection_data = text_languages(text)
+# def text_is_english_sentence(text: str) -> bool:
+#     """Determine whether or not the sentence is likely English."""
+#     language_detection_data = text_languages(text)
 
-    if language_detection_data[0]['language'] == 'en' and language_detection_data[0]['probability'] >= 0.5:
-        return True
+#     if language_detection_data[0]['language'] == 'en' and language_detection_data[0]['probability'] >= 0.5:
+#         return True
 
-    return False
+#     return False
 
 
 LEET_SPEAK_CONVERSIONS = {'1': 'i', '3': 'e', '4': 'a', '5': 's', '9': 'g', '0': 'o'}
@@ -977,8 +979,6 @@ def leet_speak_to_text(leet_speak_text):
 
 def text_to_leet_speak(text):
     """."""
-    from .strings_temp_utils import dict_flip, dict_delistify_values
-
     conversion_dict = dict_flip(LEET_SPEAK_CONVERSIONS)
     conversion_dict = dict_delistify_values(conversion_dict)
     translated_text = ''
